from IPython.core.magic import (register_line_magic,
                                register_cell_magic)
from metakernel import Magic


class GnuplotMagic(Magic):
    def __init__(self, kernel):
        """
        GnuplotMagic

        Parameters
        ----------
        kernel : GnuplotKernel
            Kernel used to execute the magic code
        """
        super(GnuplotMagic, self).__init__(kernel)

    def eval(self, code):
        """
        Evaluate code useing the gnuplot kernel
        """
        return self.kernel.do_execute_direct(code)

    def print(self, text):
        """
        Print text if it is not empty
        """
        if text and text.output.strip():
            self.kernel.Display(text)

    def line_gnuplot(self, *args):
        """
        %gnuplot CODE - evaluate code as gnuplot

        This line magic will evaluate the CODE to setup or
        unset inline plots. This magic should be used instead
        of the plot magic

        Examples:
            %gnuplot inline pngcairo enhanced transparent size 560,420
            %gnuplot inline svg enhanced size 560,420 fixed
            %gnuplot inline jpeg enhanced nointerlace
            %gnuplot qt

        """
        backend, terminal, termspec = _parse_args(args)
        terminal = terminal or 'pngcairo'
        inline_terminals = {'pngcairo': 'png',
                            'png': 'png',
                            'jpeg': 'jpg',
                            'svg': 'svg'}
        format = inline_terminals.get(terminal, 'png')

        if backend == 'inline':
            if terminal not in inline_terminals:
                msg = ("For inline plots, the terminal must be "
                       "one of pngcairo, jpeg, svg or png")
                raise ValueError(msg)

        self.kernel.plot_settings['backend'] = backend
        self.kernel.plot_settings['termspec'] = termspec
        self.kernel.plot_settings['format'] = format
        self.kernel.handle_plot_settings()

    def cell_gnuplot(self):
        """
        %%gnuplot - Run gnuplot commands

        Example:
            %%gnuplot
            unset border
            unset xtics
            g(x) = cos(2*x)/sin(x)

        Note: this is a persistent connection to a gnuplot shell.
        The working directory is synchronized to that of the notebook
        before and after each call.
        """
        result = self.eval(self.code)
        self.print(result)


def register_magics(kernel):
    """
    Make the gnuplot magic available for the GnuplotKernel
    """
    kernel.register_magics(GnuplotMagic)


def register_ipython_magics():
    """
    Register magics for the running kernel

    The magics themselve use a special kernel that
    understands gnuplot.
    """
    from ..kernel import GnuplotKernel

    # Kernel to run the both the line magic (%gnuplot)
    # and cell magic (%%gnuplot) statements
    # See: GnuplotKernel for a full notebook kernel
    kernel = GnuplotKernel()
    magic = GnuplotMagic(kernel)

    # This kernel that is used by the magics is
    # not the main kernel and it may not have access
    # to some functionality. This connects it to the
    # main kernel.
    from IPython import get_ipython
    ip = get_ipython()
    kernel.makeSubkernel(ip.parent)

    # Make magics callable:
    kernel.line_magics['gnuplot'] = magic
    kernel.cell_magics['gnuplot'] = magic

    @register_line_magic
    def gnuplot(line):
        magic.line_gnuplot(line)

    del gnuplot

    @register_cell_magic
    def gnuplot(line, cell):
        magic.code = cell
        magic.cell_gnuplot()


def _parse_args(args):
    """
    Process the gnuplot line magic arguments
    """
    if len(args) > 1:
        raise TypeError()

    sargs = args[0].split()
    backend = sargs[0]
    if backend == 'inline':
        try:
            termspec = ' '.join(sargs[1:])
            terminal = sargs[1]
        except IndexError:
            termspec = None
            terminal = None
    else:
        termspec = args[0]
        terminal = sargs[0]

    return backend, terminal, termspec



from .kernel import GnuplotKernel
from .magics import register_ipython_magics

__all__ = ['GnuplotKernel']


def load_ipython_extension(ipython):
    """
    Load the extension in IPython
    """
    register_ipython_magics()



import os


def remove_files(*filenames):
    """
    Remove the files created during the test
    """
    for filename in filenames:
        try:
            os.remove(filename)
        except FileNotFoundError:
            pass



import re
import textwrap
import signal

from metakernel import REPLWrapper
from metakernel.pexpect import TIMEOUT
from .exceptions import GnuplotError

CRLF = '\r\n'
ERROR_REs = [re.compile(r'^\s*\^\s*\n')]
NO_BLOCK = ''
PROMPT = 'gnuplot>'
PROMPT_RE = re.compile(r'^\s*gnuplot>\s*$')


class GnuplotREPLWrapper(REPLWrapper):
    # The prompt after the commands run
    prompt = ''
    _blocks = {
        'data': {
            'start_re': re.compile(r'^\$\w+\s+<<\s*(?P<end>\w+)$'),
            'end_re': re.compile(r'^(?P<end>\w+)$')
        }
    }
    _current_block = NO_BLOCK

    def exit(self):
        """
        Exit the gnuplot process
        """
        try:
            self._force_prompt(timeout=.01)
        except GnuplotError:
            return self.child.kill(signal.SIGKILL)

        self.sendline('exit')

    def is_error_output(self, text):
        """
        Return True if text is recognised as error text
        """
        for pattern in ERROR_REs:
            if pattern.match(text):
                return True
        return False

    def validate_input(self, code):
        """
        Deal with problematic input

        Raises GnuplotError if it cannot deal with it.
        """
        if code.endswith('\\'):
            raise GnuplotError("Do not execute code that "
                               "endswith backslash.")

        # Do not get stuck in the gnuplot process
        code = code.replace('\\\n', ' ')
        return code

    def send(self, cmd):
        self.child.send(cmd + '\r')

    def _force_prompt(self, timeout=30, n=4):
        """
        Force prompt
        """
        quick_timeout = .05

        if timeout < quick_timeout:
            quick_timeout = timeout

        def quick_prompt():
            try:
                self._expect_prompt(timeout=quick_timeout)
                return True
            except TIMEOUT:
                return False

        def patient_prompt():
            try:
                self._expect_prompt(timeout=timeout)
                return True
            except TIMEOUT:
                return False

        # Eagerly try to get a prompt quickly,
        # If that fails wait a while
        for i in range(n):
            if quick_prompt():
                break

            # Probably stuck in help output
            if self.child.before:
                self.send(self.child.linesep)
        else:
            # Probably long computation going on
            if not patient_prompt():
                msg = ("gnuplot prompt failed to return in "
                       "in {} seconds").format(timeout)
                raise GnuplotError(msg)

    def _end_of_block(self, stmt, end_string):
        """
        Detect the end of block statements

        Parameters
        ----------
        stmt : str
            Statement to be executed by gnuplot repl

        Returns
        -------
        end_string : str
            Terminal string for the current block.
        """
        pattern_re = self._blocks[self._current_block]['end_re']
        m = pattern_re.match(stmt)
        if m:
            if m.group('end') == end_string:
                return True
        return False

    def _start_of_block(self, stmt):
        """
        Detect the start of block statements

        Parameters
        ----------
        stmt : str
            Statement to be executed by gnuplot repl
        Returns
        -------
        block_type : str
            Name of the block that has been detected.
            Returns an empty string if none has been detected.
        end_string : str
            Terminal string for the block that has been detected.
            Returns an empty string if none has been detected.
        """
        # These are used to detect the end of the block
        block_type = NO_BLOCK
        end_string = ''
        for _type, regexps in self._blocks.items():
            m = re.match(regexps['start_re'], stmt)
            if m:
                block_type = _type
                end_string = m.group('end')
                break
        return block_type, end_string

    def _splitlines(self, code):
        """
        Split the code into lines that will be run
        """
        # Statements in a block are not followed by a prompt, this
        # confuses the repl processing. We detect a block and concatenate
        # it into single line so that after executing the line we can
        # get a prompt.
        lines = []
        block_lines = []
        end_string = ''
        stmts = code.splitlines()
        for stmt in stmts:
            if self._current_block:
                block_lines.append(stmt)
                if self._end_of_block(stmt, end_string):
                    self._current_block = NO_BLOCK
                    block_lines.append('')
                    block = '\n'.join(block_lines)
                    lines.append(block)
                    block_lines = []
                    end_string = ''
            else:
                block_name, end_string = self._start_of_block(stmt)
                if block_name:
                    self._current_block = block_name
                    block_lines.append(stmt)
                else:
                    lines.append(stmt)

        if self._current_block:
            msg = 'Error: {} block not terminated correctly.'.format(
                self._current_block)
            self._current_block = NO_BLOCK
            raise GnuplotError(msg)

        return lines

    def run_command(self, code, timeout=-1, stream_handler=None,
                    stdin_handler=None):
        """
        Run code

        This overrides the baseclass method to allow for
        input validation and error handling.
        """
        code = self.validate_input(code)

        # Split up multiline commands and feed them in bit-by-bit
        stmts = self._splitlines(code)
        output_lines = []
        for line in stmts:
            self.send(line)
            self._force_prompt()

            # Removing any crlfs makes subsequent
            # processing cleaner
            retval = self.child.before.replace(CRLF, '\n')
            self.prompt = self.child.after
            if self.is_error_output(retval):
                msg = '{}\n{}'.format(
                    line, textwrap.dedent(retval))
                raise GnuplotError(msg)

            # Sometimes block stmts like datablocks make the
            # the prompt leak into the return value
            retval = retval.replace(PROMPT,  '').strip(' ')

            # Some gnuplot installations return the input statements
            # We do not count those as output
            if retval.strip() != line.strip():
                output_lines.append(retval)

        output = ''.join(output_lines)
        return output



import sys
import re
import os.path
import uuid

from IPython.display import Image, SVG
from metakernel import MetaKernel, ProcessMetaKernel, pexpect, u
from metakernel.process_metakernel import TextOutput

from .replwrap import GnuplotREPLWrapper, PROMPT
from .exceptions import GnuplotError

# This is the only place that the version is
# specified
__version__ = '0.4.0'

# name of the command i.e first token
CMD_RE = re.compile(r'^\s*(\w+)\s?')
# "set multiplot" and abbreviated variants
MULTI_RE = re.compile(r'\s*set\s+multip(?:l|lo|lot)?')
# "unset multiplot" and abbreviated variants
UNMULTI_RE = re.compile(r'\s*uns(?:e|et)?\s+multip(?:l|lo|lot)?')
PLOT_CMDS = {
    'plot', 'plo', 'pl', 'p',
    'splot', 'splo', 'spl', 'sp',
    'replot', 'replo', 'repl', 'rep',
}
# "set output" and abbreviated variants
SET_OUTPUT_RE = re.compile(
    r'\s*set\s+o(?:u|ut|utp|utpu|utput)?(?:\s+|$)'
)

# "unset output" and abbreviated variants
UNSET_OUTPUT_RE = re.compile(
    r'\s*uns(?:e|et)?\s+o(?:u|ut|utp|utpu|utput)?\s*'
)


# funtions to recognise gnuplot statements that determine
# how we add temporary files for the images shown by jupyter
def is_set_output(stmt):
    """
    Return True if stmt is a 'set output' statement
    """
    m = re.match(SET_OUTPUT_RE, stmt)
    return True if m else False


def is_unset_output(stmt):
    """
    Return True if stmt is an 'unset output' statement
    """
    m = re.match(UNSET_OUTPUT_RE, stmt)
    return True if m else False


def is_set_multiplot(stmt):
    """
    Return True if stmt is a plot statement
    """
    m = re.match(MULTI_RE, stmt)
    return True if m else False


def is_unset_multiplot(stmt):
    """
    Return True if stmt is a plot statement
    """
    m = re.match(UNMULTI_RE, stmt)
    return True if m else False


def is_plot(stmt):
    """
    Return True if stmt is a plot statement
    """
    m = re.match(CMD_RE, stmt)
    if m:
        return m.group(1) in PLOT_CMDS
    return False


class GnuplotKernel(ProcessMetaKernel):
    implementation = 'Gnuplot Kernel'
    implementation_version = __version__
    language = 'gnuplot'
    language_version = '5.0'
    banner = 'Gnuplot Kernel'
    language_info = {
        'mimetype': 'text/x-gnuplot',
        'name': 'gnuplot',
        'file_extension': '.gp',
        'codemirror_mode': 'Octave',
        'help_links': MetaKernel.help_links,
    }
    kernel_json = {
        'argv': [sys.executable,
                 '-m', 'gnuplot_kernel',
                 '-f', '{connection_file}'],
        'display_name': 'gnuplot',
        'language': 'gnuplot',
        'name': 'gnuplot',
    }

    inline_plotting = True
    reset_code = ''
    _first = True
    _image_files = []
    _error = False

    def bad_prompt_warning(self):
        """
        Print warning if the prompt is not 'gnuplot>'
        """
        if not self.wrapper.prompt.startswith('gnuplot>'):
            msg = ("Warning: The prompt is currently set "
                   "to '{}'".format(self.wrapper.prompt))
            print(msg)

    def do_execute_direct(self, code):
        # We wrap the real function so that gnuplot_kernel can
        # give a message when an exception occurs. Without
        # this, an exception happens silently
        try:
            return self._do_execute_direct(code)
        except Exception as err:
            print(f"Error: {err}")
            raise err

    def _do_execute_direct(self, code):
        """
        Execute gnuplot code
        """
        if self._first:
            self._first = False
            self.handle_plot_settings()

        if self.inline_plotting:
            code = self.add_inline_image_statements(code)

        success = True

        try:
            result = super(GnuplotKernel,
                           self).do_execute_direct(code, silent=True)
        except GnuplotError as e:
            result = TextOutput(e.message)
            success = False

        if self.reset_code:
            super(GnuplotKernel, self).do_execute_direct(
                self.reset_code, silent=True)

        if self.inline_plotting:
            if success:
                self.display_images()
            self.delete_image_files()

        self.bad_prompt_warning()

        # No empty strings
        return result if (result and result.output) else None

    def add_inline_image_statements(self, code):
        """
        Add 'set output ...' before every plotting statement

        This is what powers inline plotting
        """
        # Ensure that there are no stale images
        self.delete_image_files()

        def set_output_inline(lines):
            filename = self.get_image_filename()
            if filename:
                lines.append("set output '{}'".format(filename))

        # We automatically create an output file for the following
        # cases if the user has not created one.
        #    - before every every plot statement that is not
        #      inside a multiplot block
        #    - before every multiplot block

        lines = []
        sm = StateMachine()
        is_joined_stmt = False
        for stmt in code.splitlines():
            sm.transition(stmt)
            add_inline_plot = (
                sm.prev_cur in (
                    ('none', 'plot'),
                    ('none', 'multiplot'),
                    ('plot', 'plot')
                )
                and not is_joined_stmt
            )
            if add_inline_plot:
                set_output_inline(lines)

            lines.append(stmt)
            is_joined_stmt = stmt.strip().endswith('\\')

        # Make gnuplot flush the output
        if not lines[-1].endswith('\\'):
            lines.append('unset output')
        code = '\n'.join(lines)
        return code

    def get_image_filename(self):
        """
        Create file to which gnuplot will write the plot

        Returns the filename.
        """
        # we could use tempfile.NamedTemporaryFile but we do not
        # want to create the file, gnuplot will create it.
        # Later on when we check if the file exists we know
        # whodunnit.
        settings = self.plot_settings
        filename = '/tmp/gnuplot-inline-{}.{}'.format(
            uuid.uuid1(),
            settings['format'])
        filename = filename
        self._image_files.append(filename)
        return filename

    def display_images(self):
        """
        Display images if gnuplot wrote to them
        """
        settings = self.plot_settings
        if self.inline_plotting:
            if settings['format'] == 'svg':
                _Image = SVG
            else:
                _Image = Image

        for filename in self._image_files:
            try:
                size = os.path.getsize(filename)
            except FileNotFoundError:
                size = 0

            if not size:
                msg = (
                    "Failed to read and display image file from gnuplot."
                    "May be you have plotted to a non interactive terminal."
                )
                print(msg)
                continue

            im = _Image(filename)
            self.Display(im)

    def delete_image_files(self):
        """
        Delete the image files
        """
        # After display_images(), the real images are
        # no longer required.
        for filename in self._image_files:
            try:
                os.remove(filename)
            except FileNotFoundError:
                pass

        self._image_files = []

    def makeWrapper(self):
        """
        Start gnuplot and return wrapper around the REPL
        """
        if pexpect.which('gnuplot'):
            program = 'gnuplot'
        elif pexpect.which('gnuplot.exe'):
            program = 'gnuplot.exe'
        else:
            raise Exception("gnuplot not found.")

        # We don't want help commands getting stuck,
        # use a non interactive PAGER
        if pexpect.which('env') and pexpect.which('cat'):
            command = 'env PAGER=cat {}'.format(program)
        else:
            command = program

        d = dict(cmd_or_spawn=command,
                 prompt_regex=u('\w*> $'),
                 prompt_change_cmd=None)
        wrapper = GnuplotREPLWrapper(**d)
        # No sleeping before sending commands to gnuplot
        wrapper.child.delaybeforesend = 0
        return wrapper

    def do_shutdown(self, restart):
        """
        Exit the gnuplot process and any other underlying stuff
        """
        self.wrapper.exit()
        super(GnuplotKernel, self).do_shutdown(restart)

    def get_kernel_help_on(self, info, level=0, none_on_fail=False):
        obj = info.get('help_obj', '')
        if not obj or len(obj.split()) > 1:
            if none_on_fail:
                return None
            else:
                return ''
        res = self.do_execute_direct('help %s' % obj)
        text = res.output.strip().rstrip(PROMPT)
        self.bad_prompt_warning()
        return text

    def handle_plot_settings(self):
        """
        Handle the current plot settings

        This is used by the gnuplot line magic. The plot magic
        is innadequate.
        """
        settings = self.plot_settings
        if ('termspec' not in settings or
                not settings['termspec']):
            settings['termspec'] = ('pngcairo size 385, 256'
                                    ' font "Arial,10"')
        if ('format' not in settings or
                not settings['format']):
            settings['format'] = 'png'

        self.inline_plotting = settings['backend'] == 'inline'

        cmd = 'set terminal {}'.format(settings['termspec'])
        self.do_execute_direct(cmd)


class StateMachine:
    """
    Track context given gnuplot statements

    This is used to help us tell when to add inline commands
    so that gnuplot can create inline images for the notebook
    """
    states = ['none', 'plot', 'output', 'multiplot', 'output_multiplot']
    previous = 'none'
    _current = 'none'

    @property
    def prev_cur(self):
        return (self.previous, self.current)

    @property
    def current(self):
        return self._current

    @current.setter
    def current(self, value):
        self.previous = self._current
        self._current = value

    def transition(self, stmt):
        lookup = {
            s: getattr(self, f'transition_from_{s}')
            for s in self.states
        }
        _transition = lookup[self.current]
        self.previous = self._current
        return _transition(stmt)

    def transition_from_plot(self, stmt):
        if self.current == 'output':
            self.current = 'none'
        elif self.current == 'plot':
            if is_plot(stmt):
                self.current = 'plot'
            elif is_set_output(stmt):
                self.current = 'output'
            else:
                self.current = 'none'

    def transition_from_none(self, stmt):
        if is_plot(stmt):
            self.current = 'plot'
        elif is_set_output(stmt):
            self.current = 'output'
        elif is_set_multiplot(stmt):
            self.current = 'multiplot'

    def transition_from_output(self, stmt):
        if is_plot(stmt):
            self.current = 'plot'
        elif is_set_multiplot(stmt):
            self.current = 'output_multiplot'
        elif is_unset_output(stmt):
            self.current = 'none'

    def transition_from_multiplot(self, stmt):
        if is_unset_multiplot(stmt):
            self.current = 'none'

    def transition_from_output_multiplot(self, stmt):
        if is_unset_multiplot(stmt):
            self.previous = self.current
            self.current = 'output'



from metakernel import Magic


class ResetMagic(Magic):

    def line_reset(self, *line):
        """
        %reset - Clear any reset

        Example:
            %reset
        """
        self.kernel.reset_code = ''

    def cell_reset(self, line):
        """
        %%reset - Change the gnuplot terminal

        This cell magic is used to specify statements that will
        silently run after every cell execution.

        Example:
            %%reset
            set key
        """
        self.kernel.reset_code = self.code


def register_magics(kernel):
    """
    Make the reset magic available for the GnuplotKernel
    """
    kernel.register_magics(ResetMagic)



import os
import weakref

from metakernel.tests.utils import (get_kernel, get_log_text,
                                    clear_log_text)
from gnuplot_kernel import GnuplotKernel
from gnuplot_kernel.magics import GnuplotMagic

from .conftest import remove_files

# Note: Empty lines after indented triple quoted may
# lead to empty statements which could obscure the
# final output.


# All kernels are registered to ensure a
# thorough cleanup
_get_kernel = get_kernel
KERNELS = weakref.WeakSet()


def get_kernel(klass=None):
    """
    Create & add to registry of live kernels
    """
    if klass:
        kernel = _get_kernel(klass)
    else:
        kernel = _get_kernel()
    KERNELS.add(kernel)
    return kernel


def teardown():
    """
    Shutdown all live kernels
    """
    while True:
        try:
            kernel = KERNELS.pop()
        except KeyError:
            break

        kernel.do_shutdown(restart=False)


# Normal workflow tests #

def test_inline_magic():
    kernel = get_kernel(GnuplotKernel)

    # gnuplot line magic changes the plot settings
    kernel.call_magic('%gnuplot pngcairo size 560, 420')
    assert kernel.plot_settings['backend'] == 'pngcairo'
    assert kernel.plot_settings['format'] == 'png'
    assert kernel.plot_settings['termspec'] == 'pngcairo size 560, 420'


def test_print():
    kernel = get_kernel(GnuplotKernel)
    code = "print cos(0)"
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert '1.0' in text


def test_file_plots():
    kernel = get_kernel(GnuplotKernel)
    kernel.call_magic('%gnuplot pngcairo size 560, 420')

    # With a non-inline terminal plot gets created
    code = """
    set output 'sine.png'
    plot sin(x)
    """
    kernel.do_execute(code)
    assert os.path.exists('sine.png')
    clear_log_text(kernel)

    # Multiple line statement
    code = """
    set output 'sine-cosine.png'
    plot sin(x),\
         cos(x)
    """
    kernel.do_execute(code)
    assert os.path.exists('sine-cosine.png')

    # Multiple line statement
    code = """
    set output 'tan.png'
    plot tan(x)
    set output 'tan2.png'
    replot
    """
    kernel.do_execute(code)
    assert os.path.exists('tan.png')
    assert os.path.exists('tan2.png')

    remove_files('sine.png', 'sine-cosine.png')
    remove_files('tan.png', 'tan2.png')


def test_inline_plots():
    kernel = get_kernel(GnuplotKernel)
    kernel.call_magic('%gnuplot inline')

    # inline plot creates data
    code = """
    plot sin(x)
    """
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert 'Display Data' in text
    clear_log_text(kernel)

    # multiple plot statements data
    code = """
    plot sin(x)
    plot cos(x)
    """
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert text.count('Display Data') == 2
    clear_log_text(kernel)

    # svg
    kernel.call_magic('%gnuplot inline svg')
    code = """
    plot tan(x)
    """
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert 'Display Data' in text
    clear_log_text(kernel)


def test_plot_abbreviations():
    kernel = get_kernel(GnuplotKernel)

    # Short names for the plot statements can be used
    # to create inline plots
    code = """
    plot sin(x)
    p cos(x)
    rep
    unset key
    sp x**2+y**2, x**2-y**2
    """
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert text.count('Display Data') == 4


def test_multiplot():
    kernel = get_kernel(GnuplotKernel)

    # multiplot
    code = """
    set multiplot layout 2,1
    plot sin(x)
    plot cos(x)
    unset multiplot
    """
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert text.count('Display Data') == 1

    # With output
    code = """
    set terminal pncairo
    set output 'multiplot-sin-cos.png'
    set multiplot layout 2, 1
    plot sin(x)
    plot cos(x)
    unset multiplot
    unset output
    """
    kernel.do_execute(code)
    assert os.path.exists('multiplot-sin-cos.png')
    remove_files('multiplot-sin-cos.png')


def test_help():
    kernel = get_kernel(GnuplotKernel)

    # The help commands should not get
    # stuck in pagers.

    # Fancy notebook help
    code = 'terminal?'
    kernel.do_execute(code)
    text = get_log_text(kernel).lower()
    assert 'subtopic' in text
    clear_log_text(kernel)

    # help by gnuplot statement
    code = 'help print'
    kernel.do_execute(code)
    text = get_log_text(kernel).lower()
    assert 'syntax' in text
    clear_log_text(kernel)


def test_badinput():
    kernel = get_kernel(GnuplotKernel)

    # No code that endswith a backslash
    code = 'plot sin(x),\\'
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert 'backslash' in text


def test_gnuplot_error_message():
    kernel = get_kernel(GnuplotKernel)

    # The error messages gets to the kernel
    code = 'plot [1,2][] sin(x)'
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert ' ^' in text


def test_bad_prompt():
    kernel = get_kernel(GnuplotKernel)
    # Anything other than 'gnuplot> '
    # is a bad prompt
    code = 'set multiplot'
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert 'warning' in text.lower()


def test_data_block():
    kernel = get_kernel(GnuplotKernel)

    # Good data block
    code = """
$DATA << EOD
# x y
1 1
2 2
3 3
4 4
EOD
plot $DATA
    """
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert text.count('Display Data') == 1
    clear_log_text(kernel)

    # Badly terminated data block
    bad_code = """
$DATA << EOD
# x y
1 1
2 2
3 3
4 4
EODX
plot $DATA
    """
    kernel.do_execute(bad_code)
    text = get_log_text(kernel)
    assert 'Error' in text
    clear_log_text(kernel)

    # Good code should work after the bad_code
    kernel.do_execute(code)
    text = get_log_text(kernel)
    assert text.count('Display Data') == 1


# magics #

def test_cell_magic():
    # To simulate '%load_ext gnuplot_kernel';
    # create a main kernel, a gnuplot kernel and
    # a gnuplot magic that uses the gnuplot kernel.
    # Then manually register the gnuplot magic into
    # the main kernel.
    kernel = get_kernel()
    gkernel = GnuplotKernel()
    gmagic = GnuplotMagic(gkernel)
    gkernel.makeSubkernel(kernel)
    kernel.line_magics['gnuplot'] = gmagic
    kernel.cell_magics['gnuplot'] = gmagic

    # inline output
    code = """%%gnuplot
    plot cos(x)
    """
    kernel.do_execute(code)
    assert 'Display Data' in get_log_text(kernel)
    clear_log_text(kernel)

    # file output
    kernel.call_magic('%gnuplot pngcairo size 560,420')
    code = """%%gnuplot
    set output 'cosine.png'
    plot cos(x)
    """
    kernel.do_execute(code)
    assert os.path.exists('cosine.png')
    clear_log_text(kernel)

    remove_files('cosine.png')


def test_reset_cell_magic():
    kernel = get_kernel(GnuplotKernel)

    # Use reset statements that have testable effect
    code = """%%reset
    set output 'sine+cosine.png'
    plot sin(x) + cos(x)
    """
    kernel.call_magic(code)
    assert not os.path.exists('sine+cosine.png')

    code = """
    unset key
    """
    kernel.do_execute(code)
    assert os.path.exists('sine+cosine.png')

    remove_files('sine+cosine.png')


def test_reset_line_magic():
    kernel = get_kernel(GnuplotKernel)

    # Create a reset
    code = """%%reset
    set output 'sine+sine.png'
    plot sin(x) + sin(x)
    """
    kernel.call_magic(code)

    # Remove the reset, execute some code and
    # make sure there are no effects
    kernel.call_magic('%reset')
    code = """
    unset key
    """
    kernel.do_execute(code)
    assert not os.path.exists('sine+sine.png')

    # Bad inline backend
    # metakernel messes this exception!!
    # with assert_raises(ValueError):
    #     kernel.call_magic('%gnuplot inline qt')


# fixture tests #
def test_remove_files():
    """
    This test create a file. Next test tests that it
    is deleted
    """
    filename = 'antigravit.txt'
    # Create file
    # make sure it exis
    with open(filename, 'w'):
        pass

    assert os.path.exists(filename)

    remove_files(filename)

    assert not os.path.exists(filename)



import io
from setuptools import find_packages, setup


__author__ = 'Hassan Kibirige'
__email__ = 'has2k1@gmail.com'
__description__ = 'A gnuplot kernel for Jupyter'
__license__ = 'BSD'
__url__ = 'https://github.com/has2k1/gnuplot_kernel'
__classifiers__ = [
    'Framework :: IPython',
    'Intended Audience :: End Users/Desktop',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: 3',
    'Topic :: Scientific/Engineering :: Visualization',
    'Topic :: System :: Shells',
]
__install_requires__ = [
    'metakernel >= 0.20.14',
    'notebook >= 5.5.0'
]
__packages__ = find_packages(
    include=['gnuplot_kernel', 'gnuplot_kernel.*']
)
__package_data__ = {'gnuplot_kernel': ['images/*.png']}

with io.open('gnuplot_kernel/kernel.py', encoding='utf-8') as fid:
    for line in fid:
        if line.startswith('__version__'):
            __version__ = line.strip().split()[-1][1:-1]
            break

with open('README.rst') as f:
    readme = f.read()

setup(name='gnuplot_kernel',
      author=__author__,
      maintainer=__author__,
      maintainer_email=__email__,
      version=__version__,
      description=__description__,
      long_description=readme,
      license=__license__,
      url=__url__,
      python_requires='>=3.6',
      install_requires=__install_requires__,
      packages=__packages__,
      package_data=__package_data__,
      classifiers=__classifiers__
      )
